#!/usr/bin/perl -w

use strict;
use warnings;
use Bio::Perl;
use Bio::SeqIO::fasta;
use Bio::SeqFeature::Generic;
use Cwd;
use Cwd qw(chdir);
use lib qw(/Users/oksana/PhD/Scripts);
use PostScript::Simple 0.07;
use File::Basename;
use Bio::SearchIO;
use Getopt::Long;
use File::Path;
use File::Glob;
use Data::Dumper;

my $start_run = time();

#############################################
########### OPTIONS, MEMORY, PATH ###########
#############################################

my ($keep, $Help, $memory, $list, $runroot, $table, $inputfile, $feed_num, $interpro_scans, $interpro, $type, $scripts_path, $prodigal_link);
my (@MEM_Pfam, @MEM_Profiles, @MEM_Superfamily);
&GetOptions (
	"keep:s" => \$keep, 
	"memory" => \$memory, 
	"feed_num=i" => \$feed_num,
	"type=s" => \$type, #genbank, assembly, proteins
	"l|list=s" => \$list,
	"f|file=s" => \$inputfile,
	"tbl=s" => \$table,
	"root=s" => \$runroot,
	"interpro_path=s" => \$interpro, 
	"interpro_scans=s" => \$interpro_scans, 
	"scripts_path=s" => \$scripts_path,
	"prodigal_link=s" => \$prodigal_link,
	"h|help"=> \$Help 
);

#print "I am here\n";

### PRINT HELP ###

print_help() if (defined $Help);


######## DEFINING OF THE OPTIONS ###########

$keep = 'PanFunPro' unless defined $keep;
$table = 'CorePanResults.txt' unless defined $table;
$feed_num = 1 unless defined $feed_num;
$interpro = '/Users/oksana/PhD/InterPro/interproscan-5-RC7' unless defined $interpro;
$interpro =~ s/\/$//;
$scripts_path = 'PanFunPro_v1.0.' unless defined $scripts_path;
$scripts_path =~ s/\/$//;
$type = 'proteins' unless defined $type;
$prodigal_link = $scripts_path."/prodigal.v2_60.pl" unless defined $prodigal_link;

## Current Working Diectory (CWD) Specification ##
# -root option defines the full path to the working directory
# if -root option is not defines, current working directory is 
# defined by getcwd(option)

my $cwd = getcwd();
$runroot = $cwd unless defined $runroot;
my $cwdpath;
if ($keep =~ m/^\//) {
	$cwdpath = $keep;
} else {	
	$cwdpath = $runroot."/".$keep;
}
mkpath($cwdpath) unless -e $cwdpath;
chdir $cwdpath;
$cwd = getcwd();


print STDERR "CWD: ".$cwd."\n";

## INTERPROSCAN PATH AND STORE DIRECTORY ##

$interpro_scans = $cwd.'/InterproScans' unless defined $interpro_scans;
mkpath($interpro_scans) unless -e $interpro_scans;


## Files used for CD-HIT clustering and results processing ##
my $clust = 'NotInHMM';
my $ClustIn = $clust.".fasta";
unlink $ClustIn;
my $ClustOut = $clust.".bak.clstr";

##################################################################################
############################### MAIN PROGRAM #####################################
##################################################################################

## Creating list of genomes to work with ##

my %FileList = ();

if (defined $inputfile) {
	system "cp $inputfile $cwd";
	my ($file, $dir) = fileparse($inputfile);
	$FileList{1}=$file;
	
} elsif (defined $list) {
	my @Config = OpenFile($list);
	for (my $i=0; $i<=$#Config; $i++) {
		print $Config[$i]."\n";
		print "CP: cp $Config[$i] $cwd\n";
		system "cp $Config[$i] $cwd";
		my ($file, $dir) = fileparse($Config[$i]);
		my $newgenome = Proteins2Prepare($type, $file, $cwd);
		$FileList{$i+1}=$newgenome;
	}
}

## RUNNING INTERPROSCAN RC3 ##
# Each proteome is scanned against PfamA, TIGRFAM, and SuperFamily databases #
# Depending on the system, different number of runs (feed_num) a time can be specified. 
# Default is 1

if (!defined $memory) {
	print STDERR "Number of scans at once: ".$feed_num."\n";
	&interpro_scan_RC7(\%FileList, $feed_num, $interpro);
}

## INTERPROSCAN, CREATION OF HMM-BASED PROFILES, FINDING GENES WITH NO HMM MATCHES ##

foreach my $item (sort keys %FileList) {
	my $file = $FileList{$item};
	print $file."\n";
	
	## PROCESSING INTERPRO RESULTS ##
	# Distribution of results into 3 groups: PfamA, TIGRFAM, Superfamily #
	
	my $InterproScan = $interpro_scans."/".$file.".Interpro.tsv";
	my $models = $interpro."/data/superfamily/1.75/model.tab";
	my (%PfamADB, %TigrfamDB, %SuperfamDB)=();
	my ($PfamADB_ref,$TigrfamDB_ref,$SuperfamDB_ref) = DB2Profiles_PW($InterproScan,$models, $file);
	%PfamADB=%{$PfamADB_ref};
	%TigrfamDB=%{$TigrfamDB_ref};
	%SuperfamDB=%{$SuperfamDB_ref};
	
	print STDERR "DOING ".$item."\n"; # Printing which genome is in process
	
	## COUNT OF GENES IN INITIAL FILE ##
	my %Genes = OpenProteinFile($file);
		
	## CREATE PROFILES FROM THE POOL OF DOMAINS ##
	#  %HMMProfiles contains genes with possible HMM-based profiles #
	#  %NotHMMProfiles contains genes, which didn't match any model in HMM collections #
	
	my ($HMMProfiles_ref, $NotHMMGenes_ref) = Profiles2Families_PW(\%Genes, \%PfamADB, \%TigrfamDB, \%SuperfamDB);
	my %HMMProfiles=%{$HMMProfiles_ref};
	my %NotHMMGenes=%{$NotHMMGenes_ref};
	
	## SAVING HMM-BASED PROFILES TO THE FILE ##
	SaveProfiles_PW($file,'hmmprofiles', %HMMProfiles);
	
	## SAVING GENES WITH NO HITS TO HMM COLLECTIONS ##
	#  Additionally join all the files into one file $ClustIn  #
	#  CD-HIT clustering will be performed on all the genes with no hits to HMM databases #
	#  
	SaveSeqs($file,'notinhmm', %NotHMMGenes);
	my $notinhmm = $file.".notinhmm";
	my $hmmprofiles = $file.".hmmprofiles";
	system "cat $notinhmm >> $ClustIn"; 
	
}

## CD-HIT CLUSTERING	##
#  Size of the window is 4 #
#  Identity cut-off is 0.6 #

system "/usr/local/bin/cd-hit -i $ClustIn -o $cwd/$clust -d 0 -c 0.6 -n 4 -g 1 -M 7800 -T 4 -bak 1";


## PARSING RESUTS OF CD-HIT CLUSTERING ##
my %CDHIT = CDHIT2Read($ClustOut);

## CORE AND PAN GENOME CALCULATION ##
#  Calculation is accumulative. Every time new genome is added, core and pan genomes are recalculated #
#  Additionally the number of new genes with addition of new genome are calculated #
#  Results are saved into $table file as tab-separated table #

my (%CoreGenome, %PanGenome, %NewGenes);

unlink $table;
open (TBLOUT, ">", $table);
print TBLOUT "Id\tGenome Name\tTotal Genes\tGenome Profiles\tCore genome\tPan Genome\tNew Profiles\n";
foreach my $item (sort {$a <=> $b} keys %FileList) { 
	my $num = $item;
	my $file = $FileList{$item};
	print "GENOME NUMBER: ".$num."\n";
	my $genomeprofiles = $file.".".$num.".profiles";
	my $notinhmm = $file.".notinhmm";
	my $hmmprofiles = $file.".hmmprofiles";
	
	my %TotalGenes = OpenProteinFile($file);
	my %NotHMMGenes = OpenProteinFile($notinhmm);
	my (%GenomeProfiles) = ();
	
	my %Clusters=CDHIT2Profiles_PW(\%CDHIT, \%NotHMMGenes);
	SaveProfiles_PW($file,'cdhitprofiles', %Clusters);
		
	my $cdhitprofiles = $file.".cdhitprofiles";
	system "cat $hmmprofiles $cdhitprofiles > $genomeprofiles";
	%GenomeProfiles = OpenProfiles_PW($genomeprofiles);
	
	if ($num == 1){
		%CoreGenome = %GenomeProfiles;
		print keys(%CoreGenome)."\t"."\t".keys(%GenomeProfiles)."\n";
		%PanGenome = %GenomeProfiles;
		%NewGenes = %GenomeProfiles;
	} elsif ($num > 1) {
		my %TMP_CoreGenome = ();
		%NewGenes=();
		my ($i, $j)=0;
		foreach my $key (keys %GenomeProfiles) {
			if (exists $CoreGenome{$key}) {
				$i++;
				foreach my $key1 (keys %{$GenomeProfiles{$key}->{'Genes'}}) {
					if (!exists $CoreGenome{$key}->{'Genes'}->{$key1}) {
						$TMP_CoreGenome{$key}->{'Genes'}->{$key1}->{'Seq'}=$GenomeProfiles{$key}->{'Genes'}->{$key1}->{'Seq'};
						$TMP_CoreGenome{$key}->{'Genes'}->{$key1}->{'Description'}=$GenomeProfiles{$key}->{'Genes'}->{$key1}->{'Description'};
					}
				}
			} elsif (!exists $CoreGenome{$key})  {
				$NewGenes{$key}->{'Genes'}=$GenomeProfiles{$key}->{'Genes'};
			}
			foreach my $key1 (keys %{$GenomeProfiles{$key}->{'Genes'}}) {
				if (!exists $PanGenome{$key}->{'Genes'}->{$key1}) {
					$PanGenome{$key}->{'Genes'}->{$key1}->{'Seq'}=$GenomeProfiles{$key}->{'Genes'}->{$key1}->{'Seq'};
					$PanGenome{$key}->{'Genes'}->{$key1}->{'Description'}=$GenomeProfiles{$key}->{'Genes'}->{$key1}->{'Description'};
				}
			}
		}
		%CoreGenome=%TMP_CoreGenome;
		print keys(%CoreGenome)."\t"."\t".keys(%GenomeProfiles)."\n";
		
	}
	
	my $num_id = $num-1;
	print TBLOUT $num_id."\t".$file."\t".keys(%TotalGenes)."\t".keys(%GenomeProfiles)."\t".keys(%CoreGenome)."\t".keys(%PanGenome)."\t".keys(%NewGenes)."\n";
}

my $end_run = time();
my $run_time = $end_run - $start_run;
open (RUN, ">>", "RUNNING_TIME.txt");
print RUN "GENOMES AT ONCE: ".$feed_num."\tJOB TOOK: ".$run_time." seconds\n";
##################################################################################
############################### SUBROUTINES ######################################
##################################################################################

sub OpenFile {
	my ($Input) = @_;
	my @Filein;
	open (IN, '<', $Input) or die "Can't read file $Input: $!";
	while (<IN>) {
		next if (/^(\s)*$/) or (/^#/);
		chomp;
		push (@Filein, $_);
	}
	close IN;
	return @Filein;
}

sub OpenProteinFile {
	my %ProteinList;
	my $fasta = Bio::SeqIO->new(-file => @_ ,-format => 'Fasta');
	while ( my $seq_object = $fasta->next_seq() ) {
		my $Seq = $seq_object->seq;
        #$Seq =~ s/\*// if $Seq =~ m/\*$/;
		my $ID = $seq_object->id;
		$ID =~ s/\|/_/g;
        my $Description = $seq_object->description;
        $Description =~ s/\"//g;
		$ProteinList{$ID}->{'Seq'}=$Seq;
		$ProteinList{$ID}->{'Description'}=$Description;
			
	}
	return %ProteinList;	
}

sub OpenProfiles_PW {
	my %ProfileList;
	my $fasta = Bio::SeqIO->new(-file => @_ ,-format => 'Fasta');
	while ( my $seq_object = $fasta->next_seq() ) {
 		my $Seq = $seq_object->seq;
		my $ID = $seq_object->id;
		$ID =~ m/(.+)\.(\d+)$/;
		my $newID = $1;
		my $Description = $seq_object->description;
		my ($GeneID, $FunctionID, $DescriptionID, $PathwayID)='';
		if ($Description =~ m/ID\=\"(.+)\"\sFunction\=\"(.+)\"\sDescription\=\"(.+)\"\sPathway\=\"(.+)\"/) {
			$GeneID = $1;
			$FunctionID = $2;
			$DescriptionID = $3;
			$PathwayID = $4; 
		}
		$ProfileList{$newID}->{'Genes'}->{$GeneID}->{'Seq'}=$Seq;
		$ProfileList{$newID}->{'Genes'}->{$GeneID}->{'Description'}=$DescriptionID;
		$ProfileList{$newID}->{'Function'}=$FunctionID;
		$ProfileList{$newID}->{'Pathway'}=$PathwayID;
	}
	return %ProfileList;	
}

sub SaveSeqs {
	my ($in_file, $extension, %Inhash) = @_;
	my $out = $in_file.".".$extension;
	open (OUT, '>', $out) or die "Can't read file $out: $!";
	foreach my $key (keys %Inhash) {
		if ($Inhash{$key}->{'Seq'} ne '') {
			if ($Inhash{$key}->{'Description'} ne '') { 
				print OUT ">".$key." ".$Inhash{$key}->{'Description'}."\n".$Inhash{$key}->{'Seq'}."\n";
			} else {
				print OUT ">".$key." ".$in_file."\n".$Inhash{$key}->{'Seq'}."\n";
			}
		} 
	}
}


sub SaveProfiles_PW {
	my ($in_file, $extension, %Inhash) = @_;
	my $out = $in_file.".".$extension;
	open (OUT, '>', $out) or die "Can't read file $out: $!";
	foreach my $key (keys %Inhash) {
		my $i=0;
		foreach my $key1 (keys %{$Inhash{$key}->{'Genes'}}) {
			print OUT ">".$key.".".$i." ID=\"".$key1."\" Function=\"".$Inhash{$key}->{'Function'}."\" Description=\"".$Inhash{$key}->{'Genes'}->{$key1}->{'Description'}."\" Pathway=\"".$Inhash{$key}->{'Pathway'}."\"\n".$Inhash{$key}->{'Genes'}->{$key1}->{'Seq'}."\n";
			$i++;
		}
	}
}

sub Proteins2Prepare {
my ($type_in, $file_in, $cwd_in) = @_;
	my $genome;
	#print "CP: cp $file_input $cwd_in\n";
	#system "cp $file_input $cwd_in";
	#my ($file_in, $dir_in) = fileparse($file_input);
	system "perl -pi -e \'s/\\*\\n/\\n/g\' $file_in";
	system "perl -pi -e \'s/\\*//g\' $file_in";
			
	$file_in =~ /(.+)(\.\w+)+$/;
	my $MainPart=$1;
	
	if ((defined $type) and ($type eq 'genbank')) {
		#print STDERR "$pathScripts/xqsub -q cge -N Prodigal_genbank -l nodes=1:ppn=1,mem=7G,walltime=24:00:00 -d $cwd -t /tools/bin/perl $pathScripts/prodigalRunner.pl --prodigal=$pathScripts/prodigal_v2_50.pl --format genbank $input";
		#system "perl $scripts_path/prodigalRunner.pl --prodigal=$scripts_path/prodigal_v2_50.pl --format genbank $file_in";
		system "perl $scripts_path/prodigalRunner.pl --prodigal=$prodigal_link --format genbank $file_in";
		$genome = $MainPart."_prodigal.orf.fsa";

	} elsif ((defined $type) and ($type eq 'assembly')) {
		$genome = $MainPart."_prodigal.orf.fsa";
		
		unlink 'Prodigal.done.out';
		system "perl $scripts_path/prodigalRunner.pl --prodigal=$prodigal_link $file_in";

		my $tmp_name = $genome."_new";
		print STDERR "Headers are going to be changed\n";
		print STDERR "NAMES ARE: ".$genome."\t".$tmp_name."\n";
		print STDERR "perl $scripts_path/Change_headders.pl -prodigal_runner -i $genome -o $tmp_name\n";
		system "perl $scripts_path/Change_headders.pl -prodigal_runner -i $genome -o $tmp_name";
		sleep 3;
		system "mv $tmp_name $genome";
		system "rm Prodigal_fasta*";
		if (-s $genome  == 0) {
			print $genome."\tPRODIGAL FAILED\n";
			die "No protein containing file is produced to proceed with analysis";
		}
	} elsif ((defined $type) and ($type eq 'proteins')) {
		$genome = $file_in;
	} else {
		die "Undefined type of the file";
	}
	return $genome;
}

sub interpro_scan_RC3 {
	my ($LOF, $num2scan, $interpro_path) = @_;
	my (@queue, @CF) = ();
	foreach my $item (sort keys %{$LOF}) {
		my $file = $LOF->{$item};
		push (@CF, $file);
	}
	for (my $i=0;$i<=$#CF || @queue > 0;) {
		if (@queue < $num2scan && $i <= $#CF) {
			my $file_in = $CF[$i];
			system "cp $file_in $interpro_path";
			system "$interpro_path/interproscan.sh -ms 1 -f TSV --iprlookup --goterms --pathways -appl PfamA,TIGRFAM,SuperFamily -i $file_in -b $file_in.Interpro";
			push (@queue, $file_in);
			sleep 10;
			$i++;
			next;
		} 
		print "Queue: ".@queue."\n";
		for (my $j=0;$j<=$#queue;) {
			my $filename = $interpro_path."/".$queue[$j].".Interpro.tsv";
			if (-e $filename) {
				splice (@queue, $j, 1);
				last;
			} 
			$j++;
		}
		sleep 10;
	}
	for (my $i=0;$i<=$#CF;$i++) {
		my $file_in=$CF[$i];
		system "cp $interpro_path/$file_in.Interpro.tsv $interpro_scans";
		system "rm $interpro_path/$file_in*";
	}
}

sub interpro_scan_RC7 {
	my ($LOF, $num2scan, $interpro_path) = @_;
	my (@queue, @CF) = ();
	foreach my $item (sort keys %{$LOF}) {
		my $file = $LOF->{$item};
		push (@CF, $file);
	}
	for (my $i=0;$i<=$#CF || @queue > 0;) {
		if (@queue < $num2scan && $i <= $#CF) {
			my $file_in = $CF[$i];
			#system "$interpro_path/interproscan.sh -ms 1 -f TSV --iprlookup --goterms --pathways -appl PfamA,TIGRFAM,SuperFamily -i $file_in -b $file_in.Interpro";
			push (@queue, $file_in);
			sleep 10;
			$i++;
			next;
		} 
		print "Queue: ".@queue."\n";
		for (my $j=0;$j<=$#queue;) {
			my $filename = $cwd."/".$queue[$j].".Interpro.tsv";
			if (-e $filename) {
				splice (@queue, $j, 1);
				last;
			} 
			$j++;
		}
		sleep 10;
	}
	for (my $i=0;$i<=$#CF;$i++) {
		my $file_in=$CF[$i];
		system "cp $file_in.Interpro.tsv $interpro_scans";
	}
}

sub DB2Profiles_PW {
	my ($scan, $inmodels, $infile) = @_;
	my @CFDB = OpenFile($scan);
	my (%PFDB,%TGDB, %SSFDB) =();
	my %SSFFuncs = FunctionsForSuperfam($inmodels);
	for (my $i=0;$i<=$#CFDB;$i++) {
		my @row=split('\t', $CFDB[$i]);
		my ($PW, $Func) = '';
		$row[5] =~ s/ /\_/g;
		$row[5] =~ s/,//g;
		$row[5] =~ s/\|/_/g;
		if ($row[5] eq '' and $row[3]) {
			if (exists $SSFFuncs{$row[4]}) {
				$Func = $SSFFuncs{$row[4]}->{'Function'};
			} 
		}
		my @row_ID = split('\|', $row[0]);
		foreach my $item (@row_ID) {
			my ($ID, $Description)='';
			my @rowDescr = split('\s', $item);
			if ($#rowDescr > 1) {
				if ($item =~ m/^(\S+)\s(.+)/) {
					$ID = $1;
					$Description = $2;
					if ($Description ne '') {
						$Description =~ s/\(//g;
						$Description =~ s/\)//g;
						$Description =~ s/\"//g;
					} else {
						$Description = $infile;
					}
				}
			} else {
				$ID = $item;
				$Description = $infile;
				$Description =~ s/\./_/g;
			}
			my @PW = ();
			if (@row > 12) {
				for (my $j=12;$j<=$#row;$j++) {
					push (@PW, $row[$j]);
				}
				$PW = join(" ",@PW);
				$PW =~ s/\s/\_/g;
			} else {
				$PW = "NA";
			}
			
			if ($row[3] eq 'Pfam') {
				$PFDB{$ID}->{'Domains'}->{$row[4]}->{'Num'}++;
				$PFDB{$ID}->{'Domains'}->{$row[4]}->{'Function'}=$row[5];
				$PFDB{$ID}->{'Domains'}->{$row[4]}->{'Pathway'}=$PW;
				$PFDB{$ID}->{'Domains'}->{$row[4]}->{$PFDB{$ID}->{'Domains'}->{$row[4]}->{'Num'}}->{'Start'}=$row[6];
				$PFDB{$ID}->{'Domains'}->{$row[4]}->{$PFDB{$ID}->{'Domains'}->{$row[4]}->{'Num'}}->{'End'}=$row[7];
				$PFDB{$ID}->{'Domains'}->{$row[4]}->{$PFDB{$ID}->{'Domains'}->{$row[4]}->{'Num'}}->{'E-value'}=$row[8];
				$PFDB{$ID}->{'MD5'}=$row[1];
				$PFDB{$ID}->{'Length'}=$row[2];
				$PFDB{$ID}->{'Description'}=$Description;	
			} elsif ($row[3] eq 'TIGRFAM') {
				$TGDB{$ID}->{'Domains'}->{$row[4]}->{'Num'}++;
				$TGDB{$ID}->{'Domains'}->{$row[4]}->{'Function'}=$row[5];
				$TGDB{$ID}->{'Domains'}->{$row[4]}->{'Pathway'}=$PW;
				$TGDB{$ID}->{'Domains'}->{$row[4]}->{$TGDB{$ID}->{'Domains'}->{$row[4]}->{'Num'}}->{'Start'}=$row[6];
				$TGDB{$ID}->{'Domains'}->{$row[4]}->{$TGDB{$ID}->{'Domains'}->{$row[4]}->{'Num'}}->{'End'}=$row[7];
				$TGDB{$ID}->{'Domains'}->{$row[4]}->{$TGDB{$ID}->{'Domains'}->{$row[4]}->{'Num'}}->{'E-value'}=$row[8];
				$TGDB{$ID}->{'MD5'}=$row[1];
				$TGDB{$ID}->{'Length'}=$row[2];
				$TGDB{$ID}->{'Description'}=$Description;	
			} elsif ($row[3] eq 'SUPERFAMILY') {
				$SSFDB{$ID}->{'Domains'}->{$row[4]}->{'Num'}++;
				$SSFDB{$ID}->{'Domains'}->{$row[4]}->{'Function'}=$row[5];
				$SSFDB{$ID}->{'Domains'}->{$row[4]}->{'Pathway'}=$PW;
				$SSFDB{$ID}->{'Domains'}->{$row[4]}->{$SSFDB{$ID}->{'Domains'}->{$row[4]}->{'Num'}}->{'Start'}=$row[6];
				$SSFDB{$ID}->{'Domains'}->{$row[4]}->{$SSFDB{$ID}->{'Domains'}->{$row[4]}->{'Num'}}->{'End'}=$row[7];
				$SSFDB{$ID}->{'Domains'}->{$row[4]}->{$SSFDB{$ID}->{'Domains'}->{$row[4]}->{'Num'}}->{'E-value'}=$row[8];
				$SSFDB{$ID}->{'MD5'}=$row[1];
				$SSFDB{$ID}->{'Length'}=$row[2];
				$SSFDB{$ID}->{'Description'}=$Description;	
			}
		}
	}
	%PFDB = JoinFeatures(%PFDB);
	%TGDB = JoinFeatures(%TGDB);
	%SSFDB = JoinFeatures(%SSFDB);
	return (\%PFDB, \%TGDB, \%SSFDB);
}

sub JoinFeatures {
	my (%DB) = @_;	
	foreach my $key (keys %DB) {
		my (@Accs, @Funcs, @PWs)=();
		my (%InDBPart, %InDBPart1)=();
		foreach my $key1 (keys %{$DB{$key}->{'Domains'}}) {
			$InDBPart{$key1}=$DB{$key}->{'Domains'}->{$key1}->{'Function'};
			$InDBPart1{$key1}=$DB{$key}->{'Domains'}->{$key1}->{'Pathway'};
		}
		foreach my $key2 (sort keys %InDBPart) {
			push (@Accs, $key2);
			push (@Funcs, $InDBPart{$key2});
		}
		foreach my $key3 (sort keys %InDBPart1) {
			push (@PWs, $InDBPart1{$key3});
		}
		my $Pathway = join('||', @PWs);
		my $Profile = join('_', @Accs);
		my $Func = join('||', @Funcs);
		$DB{$key}->{'Function'}=$Func;
		$DB{$key}->{'Profile'}=$Profile;
		$DB{$key}->{'Pathway'}=$Pathway;
	}
	return %DB;
}

sub Profiles2Families_PW {
	my ($Fasta, $DB1, $DB2, $DB3) = @_;
	my (%HMMhash, %NotHMMhash);
	foreach my $key (keys %{$Fasta}){
		if (exists $DB1->{$key}) {
			$HMMhash{$DB1->{$key}->{'Profile'}}->{'Function'}=$DB1->{$key}->{'Function'};
			$HMMhash{$DB1->{$key}->{'Profile'}}->{'Pathway'}=$DB1->{$key}->{'Pathway'};
			$HMMhash{$DB1->{$key}->{'Profile'}}->{'Genes'}->{$key}->{'Seq'}=$Fasta->{$key}->{'Seq'};
			$HMMhash{$DB1->{$key}->{'Profile'}}->{'Genes'}->{$key}->{'Description'}=$DB1->{$key}->{'Description'};
			$HMMhash{$DB1->{$key}->{'Profile'}}->{'Genes'}->{$key}->{'Length'}=$DB1->{$key}->{'Length'};
			$HMMhash{$DB1->{$key}->{'Profile'}}->{'Genes'}->{$key}->{'MD5'}=$DB1->{$key}->{'MD5'};
		} elsif (exists $DB2->{$key}) {
			$HMMhash{$DB2->{$key}->{'Profile'}}->{'Function'}=$DB2->{$key}->{'Function'};
			$HMMhash{$DB2->{$key}->{'Profile'}}->{'Pathway'}=$DB2->{$key}->{'Pathway'};
			$HMMhash{$DB2->{$key}->{'Profile'}}->{'Genes'}->{$key}->{'Seq'}=$Fasta->{$key}->{'Seq'};
			$HMMhash{$DB2->{$key}->{'Profile'}}->{'Genes'}->{$key}->{'Description'}=$DB2->{$key}->{'Description'};
			$HMMhash{$DB2->{$key}->{'Profile'}}->{'Genes'}->{$key}->{'Length'}=$DB2->{$key}->{'Length'};
			$HMMhash{$DB2->{$key}->{'Profile'}}->{'Genes'}->{$key}->{'MD5'}=$DB2->{$key}->{'MD5'};
		} elsif (exists $DB3->{$key}) {
			$HMMhash{$DB3->{$key}->{'Profile'}}->{'Function'}=$DB3->{$key}->{'Function'};
			$HMMhash{$DB3->{$key}->{'Profile'}}->{'Pathway'}=$DB3->{$key}->{'Pathway'};
			$HMMhash{$DB3->{$key}->{'Profile'}}->{'Genes'}->{$key}->{'Seq'}=$Fasta->{$key}->{'Seq'};
			$HMMhash{$DB3->{$key}->{'Profile'}}->{'Genes'}->{$key}->{'Description'}=$DB3->{$key}->{'Description'};
			$HMMhash{$DB3->{$key}->{'Profile'}}->{'Genes'}->{$key}->{'Length'}=$DB3->{$key}->{'Length'};
			$HMMhash{$DB3->{$key}->{'Profile'}}->{'Genes'}->{$key}->{'MD5'}=$DB3->{$key}->{'MD5'}; 
		} else {
			$NotHMMhash{$key}->{'Seq'}=$Fasta->{$key}->{'Seq'};
			$NotHMMhash{$key}->{'Description'}=$Fasta->{$key}->{'Description'};
		}
	}
	return (\%HMMhash, \%NotHMMhash);
}

sub CDHIT2Read {
	my @ClusteringOut = OpenFile(@_);
	my (%ClusterToGene);
	for (my $i=0; $i<=$#ClusteringOut; $i++) {
		if ($ClusteringOut[$i] =~ m/^(\d+)\t(\d+)aa,\s>(.+)\.\.\..+/) {
			$ClusterToGene{$3}->{'Cluster'}=$1;
			$ClusterToGene{$3}->{'Length'}=$2;
		} 
	}
	return %ClusterToGene;
}

sub CDHIT2Profiles_PW { 
	my ($ClusterToGene, $NotHMMhash) = @_;
	my (%Cluster2Profile);
	#foreach my $key (keys %ClusterToGene) {
		foreach my $key (keys %{$NotHMMhash}) {
			if (exists $ClusterToGene->{$key}) {
				my $clustername = "CL".$ClusterToGene->{$key}->{'Cluster'};
				my $func = 'Cluster '.$clustername;
				$Cluster2Profile{$clustername}->{'Function'} = $func;
				$Cluster2Profile{$clustername}->{'Pathway'} = 'NA';
				$Cluster2Profile{$clustername}->{'Genes'}->{$key}->{'Description'}=$NotHMMhash->{$key}->{'Description'};
				$Cluster2Profile{$clustername}->{'Genes'}->{$key}->{'Seq'}=$NotHMMhash->{$key}->{'Seq'};
				$Cluster2Profile{$clustername}->{'Genes'}->{$key}->{'Length'}=$ClusterToGene->{$key}->{'Length'};
			}
	}
	return %Cluster2Profile;
}

sub FunctionsForSuperfam {
	my ($inmodels, %DB)=@_;
	my %SuperFunctions;
	my @ModelCF = OpenFile($inmodels);
	for (my $i=0;$i<=$#ModelCF;$i++) {
		my @row = split('\t', $ModelCF[$i]);
		my $Domain = "SSF".$row[1];
		$SuperFunctions{$Domain}->{'Function'}=$row[4];
	}
	return %SuperFunctions;
}

sub SplitFile {
	my ($input, $seqs) = @_;
	my $count = 0;
	my $Part = 0;
	my %NewHash = ();
	my %Genome = OpenProteinFile($input);
	my $filename;
	my @PartsList;
	foreach my $key (keys %Genome) {
		$count ++;
		$NewHash{$key} = $Genome{$key};
		if ($count == $seqs) {
			$Part ++;
			&SaveFasta($input, $Part, %NewHash);
			$filename = $input.".".$Part.".fsa";
			push(@PartsList, $filename);
			$count = 0;
			%NewHash = ();
		}
	}
	$Part ++;
	&SaveFasta($input, $Part, %NewHash);
	$filename = $input.".".$Part.".fsa";
	push(@PartsList, $filename);
	return @PartsList;
}

sub SaveFasta {
	my ($input, $extension, %Hash_Part) = @_;
	my $filename = $input.".".$extension.".fsa";
	unlink ($filename);
	open (OUT, ">",  $filename) or die "Can't read file $filename: $!";
	foreach my $key (sort keys %Hash_Part) {
		print OUT ">$key ".$Hash_Part{$key}->{'Description'}."\n".$Hash_Part{$key}->{'Seq'}."\n";
	}
	close OUT;
}

sub pancoreplot {
	my ($tbl, $panplot) = @_;
	open R , "| /tools/bin/R-2.12 --vanilla > /dev/null";
	print R "
postscript('$cwd/$panplot');
layout( matrix(c(1,2), 1, 2, byrow = TRUE),widths=c(0.7,0.3))
data <- read.table('$tbl',skip=1,sep='\t',dec='.',header=FALSE);
tN <- data[,3]
x<-rbind(data[,4],data[,7])
r <- barplot(x,beside=TRUE,cex.names=0.7,names.arg=as.vector(data[,1])+1,ylim=c(0,1.3*max(data[,6])))
lines(r[2,], type='b', data[,5], col='red', lwd=3, cex=1)
lines(r[2,], type='b', data[,6], col='blue', lwd=3, cex=1)
legend(1,1.3*max(data[,6]),c('Total gene families','New gene families','Core genome','Pan genome'),col=c('darkgray','lightgray','red','blue'), lwd=c(4,4,4,4))
plot.new()
plot.window(xlim=c(0,2),ylim=c(-nrow(data),1))
for (i in 1:nrow(data)) {
 text(0,-i, adj=0,paste(i,': ',data[i,2]),cex=0.7)
}
dev.off();
";
	close R;
}

sub matrix {
	my ($tbl, $mat, $LOF) = @_;
	my (%CMP, %displayMin, %displayMax);
	$displayMax{"paralogs"}=0;
	$displayMax{"orthologs"}=0;
	$displayMin{"paralogs"}=1;
	$displayMin{"orthologs"}=1;
	foreach my $item (sort {$a <=> $b} keys %{$LOF}) { 
		my $file = $LOF->{$item};
		my $genomeprofiles = $file.".".$item.".profiles";
		my %QueryGenes = OpenProteinFile($file);
		my %QueryProfiles = OpenProfiles_PW($genomeprofiles);
		my $num1 = $item-1;
		$CMP{$num1}->{'Profiles'}=keys(%QueryProfiles);
		$CMP{$num1}->{'Name'}=$LOF->{$item};
		$CMP{$num1}->{'Genes'}=keys(%QueryGenes);
		foreach my $item1 (sort {$a <=> $b} keys %FileList) {
			print STDERR "PAIR: ".$item." - ".$item1."\n";
			my $num2 = $item1-1;
			if ($item1 eq $item) {
				my $common = WithinGenome(\%QueryProfiles);
				my $within = $common/keys(%QueryProfiles);
				my $within_rn = sprintf("%.1f", $within*100);
				$CMP{$num1}->{'Genomes'}->{$num2}->{'Fraction'}=$within;
				$CMP{$num1}->{'Genomes'}->{$num2}->{'Total'}=$CMP{$num1}->{'Profiles'};
				$CMP{$num1}->{'Genomes'}->{$num2}->{'Name'}=$FileList{$item1};
				$CMP{$num1}->{'Genomes'}->{$num2}->{'Common'}=$common;
				$CMP{$num1}->{'Genomes'}->{$num2}->{'FractionText'}=$within_rn;
				$CMP{$num1}->{'Genomes'}->{$num2}->{'Type'}='paralogs';
				if ($within < $displayMin{"paralogs"}) {
					$displayMin{"paralogs"} = $within;
				}
				if ($within > $displayMax{"paralogs"}) {
					$displayMax{"paralogs"} = $within;
				}
			} else {
				if (exists $CMP{$num2}->{'Genomes'}->{$num1}) {
					$CMP{$num1}->{'Genomes'}->{$num2}->{'Total'}=$CMP{$num2}->{'Genomes'}->{$num1}->{'Total'};
					$CMP{$num1}->{'Genomes'}->{$num2}->{'Name'}=$FileList{$item1};
					$CMP{$num1}->{'Genomes'}->{$num2}->{'Common'}=$CMP{$num2}->{'Genomes'}->{$num1}->{'Common'};
					$CMP{$num1}->{'Genomes'}->{$num2}->{'Fraction'}=$CMP{$num2}->{'Genomes'}->{$num1}->{'Fraction'};
					$CMP{$num1}->{'Genomes'}->{$num2}->{'FractionText'}=$CMP{$num2}->{'Genomes'}->{$num1}->{'FractionText'};
					$CMP{$num1}->{'Genomes'}->{$num2}->{'Type'}='orthologs';
					if ($CMP{$num1}->{'Genomes'}->{$num2}->{'Fraction'} < $displayMin{"orthologs"}) {
						$displayMin{"orthologs"} = $CMP{$num1}->{'Genomes'}->{$num2}->{'Fraction'};
					}
					if ($CMP{$num1}->{'Genomes'}->{$num2}->{'Fraction'} > $displayMax{"orthologs"}) {
						$displayMax{"orthologs"} = $CMP{$num1}->{'Genomes'}->{$num2}->{'Fraction'};
					}
				} else {
					my $file1 = $FileList{$item1};
					my %SubjectGenes = OpenProteinFile($file1);
					my $SubjectProfiles = $file1.".".$item1.".profiles";
					my %SubjectProfiles = OpenProfiles_PW($SubjectProfiles);
					my @Res_cmp = ComparePair(\%QueryProfiles, \%SubjectProfiles);
					my $core = $Res_cmp[0];
					my $pan = $Res_cmp[1];
					my $cmp = $core/$pan;
					my $cmp_rn = sprintf("%.1f", $cmp*100);
					$CMP{$num1}->{'Genomes'}->{$num2}->{'Total'}=$pan;
					$CMP{$num1}->{'Genomes'}->{$num2}->{'Name'}=$FileList{$item1};
					$CMP{$num1}->{'Genomes'}->{$num2}->{'Common'}=$core;
					$CMP{$num1}->{'Genomes'}->{$num2}->{'Fraction'}=$cmp;
					$CMP{$num1}->{'Genomes'}->{$num2}->{'FractionText'}=$cmp_rn;
					$CMP{$num1}->{'Genomes'}->{$num2}->{'Type'}='orthologs';
					if ($cmp < $displayMin{"orthologs"}) {
						$displayMin{"orthologs"} = $cmp;
					}
					if ($cmp > $displayMax{"orthologs"}) {
						$displayMax{"orthologs"} = $cmp;
					}
				}
			}
		}
	}
	&PrintPS(\%CMP, \%displayMin, \%displayMax);
}


sub WithinGenome {
	my ($Profiles)=@_;
	my $common;
	foreach my $key (keys %{$Profiles}) {
		if (keys (%{$Profiles->{$key}->{'Genes'}}) > 1) {
			$common++;
		}
	}
	return $common;
}

sub ComparePair {
	my ($Profiles1, $Profiles2)=@_;
	my @results;
	my %Total = (%{$Profiles1}, %{$Profiles2});
	my %Common = Core(\%{$Profiles1}, \%{$Profiles2});
	my $total = keys(%Total);
	my $common = keys(%Common);
	push (@results, $common, $total);
	return @results;
}

sub Core {
	my ($Profiles1, $Profiles2) = @_;
	my %Common;
	foreach my $key (keys %{$Profiles2}) {
		if (exists $Profiles1->{$key}) {
			$Common{$key}=$Profiles1;
		}
	}
	return %Common;
}

sub PrintMatrix {
	my ($Results, $fileout) = @_;
	open (OUT, '>', $fileout) or die "Can't read file $fileout: $!";
	print OUT "GENOMES";
;	foreach my $key (sort {$a <=> $b} keys %{$Results}) {
		print OUT "\t".$Results->{$key}->{'Name'};
	}
	print OUT "\n";
	foreach my $key (sort {$a <=> $b} keys %{$Results}) {
		print OUT $Results->{$key}->{'Name'};
		my @line = ();
		for my $item (0 .. keys (%{$Results->{$key}->{'Genomes'}})) {
			push (@line, $Results->{$key}->{'Genomes'}->{$item}->{'FractionText'});
		}
		my $line = join ("\t", @line);
		print OUT "\t".$line."\n";
	}
}


sub PrintPS {
	my ($CMP, $displayMn, $displayMx) = @_;
	my $large = 1; # used with large number of proteomes (~50)

	my $DIM = keys(%{$CMP});
	# figure out if groups are being drawn
	my $draw_groups;
	
	my $horzVertTextdim = 40 ; # space assigned for text ADJUST HERE!!!!!
	$horzVertTextdim = 120 if defined $draw_groups;

	my $vertTitleSpace = 30;
	my $legendHeight = 6;
	$legendHeight = 12 if $large;
	my $legendWidth = 80;
	$legendWidth = 160 if $large;
	
	my $legendSteps = 20;
	my $legendMatrixDistance = -20;
	my @LEFT_RIGHT_MARGIN = (10,10);
	my @TOP_BOTTOM_MARGIN = (10,10);
	my $BOX_SIZE = 20;

	# obscure!!!!
	my $R = 1.6;
	my $R2 = 3;
	my $V2 = 0.5 * sqrt ( ($BOX_SIZE-$R)**2 + ($BOX_SIZE-$R)**2 );
	my $V5 = 0.5 * sqrt ( ($BOX_SIZE-$R2)**2 + ($BOX_SIZE-$R2)**2 );
	my $V = 0.5 * sqrt ( $BOX_SIZE**2 + $BOX_SIZE**2 );
	my $V3 = sqrt ( $R**2 + $R**2 );
	my $V4 = sqrt ( $R2**2 / 2 ) * 1.1;
	my $legendDist = 15;
	# paper size
	my $XSIZE = $legendMatrixDistance + ($legendDist + $legendWidth)  + $horzVertTextdim + sqrt ( $BOX_SIZE**2 + $BOX_SIZE**2 ) * $DIM + $LEFT_RIGHT_MARGIN[0] + $LEFT_RIGHT_MARGIN[1];
	my $YSIZE = $horzVertTextdim + $V * ($DIM + 0.5 ) + $TOP_BOTTOM_MARGIN[0] + $TOP_BOTTOM_MARGIN[1] + $vertTitleSpace + $legendHeight; 

	my @TOP = 
		($LEFT_RIGHT_MARGIN[0]+$horzVertTextdim + 0.5 * sqrt ( $BOX_SIZE**2 + $BOX_SIZE**2 ) * $DIM , 
		$YSIZE - $TOP_BOTTOM_MARGIN[0] - $horzVertTextdim
	);

	my $p = new PostScript::Simple(
	 xsize => $XSIZE,
	 ysize => $YSIZE,
	 colour => 1,
	 eps => 1,
	 units => "mm"
	);
	my $grp_dist = 70;
	my $last_group;
	$p->setfont("Palatino-Italic", 10);
	foreach my $s ( 1 .. $DIM  ) {
		foreach my $q ( $s .. $DIM  ) {
			my @top = ($TOP[0] + ($DIM-$q)*$V - ($s-1) * $V  , $TOP[1] - ($s-1) * $V - ($DIM - $q) * $V);
			my $type = "orthologs";
			$type = "paralogs" if $s == $q;
			my $fraction = $CMP->{$s-1}->{'Genomes'}->{$q-1}->{'Fraction'};
			$p->setcolour ((score2RGB($type,fraction2score($type,$fraction, \%{$displayMx}, \%{$displayMn}))));
			my @BOX;
			if ( $s == $q ) {
				@BOX = ( $top[0] , $top[1] , $top[0] - $V2, $top[1]-$V2 , $top[0] , $top[1]-2*$V2 , $top[0]+$V2 , $top[1]-$V2);
				$p->polygon({filled=>1},@BOX);
				my @BOX = (
					$top[0] - $V          , $top[1] - $V , 
					$top[0] - $V + $V4    , $top[1] - $V + $V4 , 
					$top[0]               , $top[1] - 2 * $V + 2 * $V4 ,
					$top[0] - $V4         , $top[1] - 2 * $V + $V4
				);
				$p->polygon({filled=>1},@BOX);
				@BOX = (
					$top[0] + $V          , $top[1] - $V , 
					$top[0] + $V - $V4    , $top[1] - $V + $V4 , 
					$top[0]               , $top[1] - 2 * $V + 2 * $V4 ,
					$top[0] + $V4         , $top[1] - 2 * $V + $V4 
			);
				$p->polygon({filled=>1},@BOX);
				$p->circle( {filled=>1} ,  $top[0] , $top[1]-2*$V5, $R2 );
			} else {
				@BOX = ( $top[0] , $top[1] , $top[0] - $V , $top[1]-$V , $top[0] , $top[1]-2*$V , $top[0]+$V , $top[1]-$V);
				$p->polygon({filled=>1},@BOX);
			}
			$p->setcolour ( 0,0,0);
			$p->setfont("Palatino-Bolditalic", 10);
			$p->text({rotate=>'0',align=>'center'},($top[0],$top[1]-$V+4), ,$CMP->{$q-1}->{'Genomes'}->{$s-1}->{'FractionText'}." %" );
			$p->setfont("Palatino-Italic", 10);
			my $sharedfamilies = $CMP->{$q-1}->{'Genomes'}->{$s-1}->{'Common'};
			$sharedfamilies =~ s/(\d{1,3}?)(?=(\d{3})+$)/$1,/g;
			my $totalfamilies = $CMP->{$q-1}->{'Genomes'}->{$s-1}->{'Total'};
			$totalfamilies =~ s/(\d{1,3}?)(?=(\d{3})+$)/$1,/g;
			$p->text({rotate=>'0',align=>'center'},($top[0],$top[1]-$V-1), ,"$sharedfamilies / $totalfamilies" );
		}
	}
	foreach my $s ( 1 .. $DIM  ) {
		foreach my $q ( $s .. $DIM  ) {
			my @top = ($TOP[0] - ($DIM-$q) * $V + ($s-1) * $V  , $TOP[1] - ($s-1) * $V - ($DIM - $q) * $V);
			$p->setcolour ( 255,255,255);
			$p->circle( {filled=>1} , $top[0] , $top[1] , $R );
		}
	}
	my %GROUPS;
	foreach my $q ( 1 .. $DIM  ) {
		$p->setcolour ( 0,0,0);
		$p->setfont("Palatino-Italic", 18);
		$p->text({rotate=>'-45',align=>'right'},($TOP[0] - ($q-1) * $V - 0.7*$V, $TOP[1] - ($q-1) * $V), ,$CMP->{$q-1}->{'Name'} );
		$p->setfont("Palatino-Roman", 10);
		$p->text({rotate=>'-45',align=>'right'},($TOP[0] - ($q-1) * $V - 0.5*$V - 5 - 1 , $TOP[1] - ($q-1) * $V - 5 + 1), ,$CMP->{$q-1}->{'Name'} );
		my $genes = $CMP->{$q-1}->{'Genes'};
		$genes =~ s/(\d{1,3}?)(?=(\d{3})+$)/$1,/g;
		my $totalfamilies = $CMP->{$q-1}->{'Genomes'}->{$q-1}->{'Total'};
		$totalfamilies =~ s/(\d{1,3}?)(?=(\d{3})+$)/$1,/g;
		$p->text({rotate=>'-45',align=>'right'},($TOP[0] - ($q-1) * $V - 0.5*$V - 8 - 1, $TOP[1] - ($q-1) * $V - 8 + 1 ), ,"$genes proteins, $totalfamilies families" );
	}
	my $Q = 0; # used to position titles and color boxes
	foreach my $s ( 1 .. $DIM  ) {
		$p->setcolour ( 0 , 0 , 0 );
		$p->setfont("Palatino-Italic", 18);
		$p->text({rotate=>'45',align=>'left'},($TOP[0] + ($DIM-$s) * $V + 0.7*$V,$TOP[1] - ($DIM-$s) * $V), ,$CMP->{$s-1}->{'Name'} );
		$p->setfont("Palatino-Roman", 10);
		$p->text({rotate=>'45',align=>'left'},($TOP[0] + ($DIM-$s) * $V + 0.5*$V + 5 + 1  ,$TOP[1] - ($DIM-$s) * $V - 5  + 1 ), ,$CMP->{$s-1}->{'Name'} );
		my $genes = $CMP->{$s-1}->{'Genes'};
		$genes =~ s/(\d{1,3}?)(?=(\d{3})+$)/$1,/g;
		my $totalfamilies = $CMP->{$s-1}->{'Genomes'}->{$s-1}->{'Total'};
		$totalfamilies =~ s/(\d{1,3}?)(?=(\d{3})+$)/$1,/g;
		$p->text({rotate=>'45',align=>'left'},($TOP[0] + ($DIM-$s) * $V + 0.5*$V + 8 + 1 ,$TOP[1] - ($DIM-$s) * $V - 8 + 1), ,"$genes proteins, $totalfamilies families" );
	}

	my $inc = 1/$legendSteps;
	my $legendBoxWidth = $legendWidth / $legendSteps;
	my $legendYpos_paralogs = $TOP_BOTTOM_MARGIN[1]+8;
	my $legendYpos_orthlogs = $YSIZE - $TOP_BOTTOM_MARGIN[0] - 12 - $legendHeight;
	my $legendXpos = $XSIZE - $LEFT_RIGHT_MARGIN[1] - $legendWidth - $legendDist;
	$p->setcolour ( 0 , 0 , 0 );

	$p->setfont("Palatino-Roman", 18);
	$p->setfont("Palatino-Roman", 24) if $large;

	$p->text({rotate=>'0',align=>'center'}, $legendXpos + $legendWidth / 2 , $legendYpos_paralogs+$legendHeight+2 , ,"Homology within proteomes");
	$p->setfont("Palatino-Roman", 18);
	$p->setfont("Palatino-Roman", 24) if $large;
	$p->text({rotate=>'0',align=>'center'},( $legendXpos + $legendWidth), $legendYpos_paralogs-9 , ,sprintf('%0.1f',(100*$displayMx->{"paralogs"})).' %');
	$p->text({rotate=>'0',align=>'center'}, $legendXpos, $legendYpos_paralogs-9 , ,sprintf('%0.1f',(100*$displayMn->{"paralogs"})).' %');
	for (my $i = 0 ; $i <= 1 ; $i+=$inc) {
		$p->setcolour ((score2RGB ("paralogs",$i)));
		my @BOX = (
			$legendXpos + (($i ) * $legendWidth) , $legendYpos_paralogs , 
			$legendXpos + (($i ) * $legendWidth), $legendYpos_paralogs+$legendHeight , 
			$legendXpos + (($i + $inc )* $legendWidth), $legendYpos_paralogs+$legendHeight , 
			$legendXpos + (($i + $inc ) * $legendWidth), $legendYpos_paralogs 
		);
		$p->polygon({filled=>1},@BOX);
	}
	$p->setcolour ( 0,0,0);
	$p->setfont("Palatino-Bold", 22);
	$p->setfont("Palatino-Roman", 18);
	$p->setcolour ( 0,0,0);
	$p->setfont("Palatino-Roman", 18);
	$p->setfont("Palatino-Roman", 24) if $large;
	$p->text({rotate=>'0',align=>'center'},$legendXpos + $legendWidth / 2 , $legendYpos_orthlogs+$legendHeight+2 , ,"Homology between proteomes");
	$p->setfont("Palatino-Roman", 18);
	$p->setfont("Palatino-Roman", 24) if $large;
	$p->text({rotate=>'0',align=>'center'},($legendXpos + $legendWidth), $legendYpos_orthlogs-9 , ,sprintf('%0.1f',100*$displayMx->{"orthologs"}).' %');
	$p->text({rotate=>'0',align=>'center'},$legendXpos, $legendYpos_orthlogs-9 , ,sprintf('%0.1f',100*$displayMn->{"orthologs"}).' %');
	for (my $i = 0 ; $i <= 1 ; $i+=$inc) {
		$p->setcolour ( (  score2RGB("orthologs",$i)));
		my @BOX = (
			$legendXpos + (($i ) * $legendWidth) , $legendYpos_orthlogs , 
			$legendXpos + (($i ) * $legendWidth), $legendYpos_orthlogs+$legendHeight , 
			$legendXpos + (($i + $inc )* $legendWidth), $legendYpos_orthlogs+$legendHeight , 
			$legendXpos + (($i + $inc ) * $legendWidth), $legendYpos_orthlogs 
		);
		$p->polygon({filled=>1},@BOX);
	}
	$p->setcolour ( 200,200,200);
	$p->setlinewidth(1.5);
	$p->line ( 
	$XSIZE-$LEFT_RIGHT_MARGIN[1] - 4  , $YSIZE-$TOP_BOTTOM_MARGIN[0] , 
	$XSIZE-$LEFT_RIGHT_MARGIN[1]  ,     $YSIZE-$TOP_BOTTOM_MARGIN[0] 
	);
	$p->linextend ( 
		$XSIZE-$LEFT_RIGHT_MARGIN[1] , $YSIZE - $TOP_BOTTOM_MARGIN[0] -  $horzVertTextdim - $V * $DIM + 1 
	);
	$p->linextend ( 
		$XSIZE-$LEFT_RIGHT_MARGIN[1] -12  , $YSIZE - $TOP_BOTTOM_MARGIN[0] -  $horzVertTextdim - $V * $DIM + 1 
	);
	$p->line ( 
		$XSIZE-$LEFT_RIGHT_MARGIN[1] -12  ,  $YSIZE - $TOP_BOTTOM_MARGIN[0] -  $horzVertTextdim - $V * $DIM - 1, 
		$XSIZE-$LEFT_RIGHT_MARGIN[1]  ,  $YSIZE - $TOP_BOTTOM_MARGIN[0] -  $horzVertTextdim - $V * $DIM - 1 
	);
	$p->linextend ( 
		$XSIZE-$LEFT_RIGHT_MARGIN[1]  , $TOP_BOTTOM_MARGIN[1]
	) ;
	$p->linextend ( 
		$XSIZE-$LEFT_RIGHT_MARGIN[1] -4 , $TOP_BOTTOM_MARGIN[1]
	) ;
	$p->output("$cwd/PanFunPro_matrix.ps");
}

sub fraction2score {
	my ($type_in , $fraction_in, $displayMx, $displayMn) = @_;
	my $slope = 1/($displayMx->{$type_in} - $displayMn->{$type_in});
	my $intercept = 1 - ($slope * $displayMx->{$type_in});
	my $score = ($fraction_in*$slope) + $intercept;
	return $score;
}

sub score2RGB {
	# for instance 'paralogs'
	my ( $type_in , $score) = @_;
	my %Colours = &Colours;
	$score = 0 if $score < 0;
	$score = 1 if $score > 1;
	my @startColors = ($Colours{$type_in}->{'from_color'}->{'r'},$Colours{$type_in}->{'from_color'}->{'g'},$Colours{$type_in}->{'from_color'}->{'b'});
	my @endColors = ($Colours{$type_in}->{'to_color'}->{'r'},$Colours{$type_in}->{'to_color'}->{'g'},$Colours{$type_in}->{'to_color'}->{'b'});
	my @resultColor;
	foreach my $x (0 .. 2) {
		$resultColor[$x] = sprintf ( "%0.1f" ,( $startColors[$x] + ($endColors[$x] - $startColors[$x]) * $score) * 256);
	}
	return @resultColor;
}

sub Colours {
	my %Col;
	$Col{'orthologs'}->{'range_from'}='auto';
	$Col{"orthologs"}->{'range_to'}='auto';
	$Col{"orthologs"}->{'to_color'}->{'r'}=0;
	$Col{"orthologs"}->{'to_color'}->{'b'}=0.975;
	$Col{"orthologs"}->{'to_color'}->{'g'}=0.8;
	$Col{"orthologs"}->{'from_color'}->{'r'}=0.9;
	$Col{"orthologs"}->{'from_color'}->{'b'}=0.9;
	$Col{"orthologs"}->{'from_color'}->{'g'}=0.9;
	$Col{"paralogs"}->{'range_from'}='auto';
	$Col{"paralogs"}->{'range_to'}='auto';
	$Col{"paralogs"}->{'to_color'}->{'r'}=0.975;
	$Col{"paralogs"}->{'to_color'}->{'b'}=0;
	$Col{"paralogs"}->{'to_color'}->{'g'}=0;
	$Col{"paralogs"}->{'from_color'}->{'r'}=0.9;
	$Col{"paralogs"}->{'from_color'}->{'b'}=0.9;
	$Col{"paralogs"}->{'from_color'}->{'g'}=0.9;
	return %Col;
}

# --------------------------------------------------------------------
# %% Help Page/Documentation %%
#

sub print_help {
  print <<END;

NAME
	PanFunPro - groups genes of each genome into gene families, based on PfamA, TIGRFAM, Superfamily HMM collections. 
	Results can displayed either with Pan-/Core plot or PanFunPro matrix. 
   

SYNOPSIS
    perl PanFunPro2profiles.pl [-list] [-keep] [-file] [-memory] [-tbl] [-root] [-h] 

DESCRIPTION
	PanFunPro reads a list of proteomes from the file, defining the proteomes to be compared. 
	The order of the list indicate in what order the genomes will appear in the pancoreplot
	or matrix. Each line must contain data source (full path to the ). Example:
	
	
	/home/peope/oksana/Genomes/PRJNA57623_prodigal.orf.fsa
	/home/peope/oksana/Genomes/PRJNA57623.fsa
	/home/peope/oksana/Genomes/PRJNA57623.proteins.fsa
	
    Each proteome is scanned against three collections of HMMs: PfamA, TIGRFAM, and Superfamily. Scan is performed 
    using Interpro software. For each gene functional profiles are constructed. Genes, which didn't find any match 
    in any of these databases, were clustered using CD-HIT tool. Results for each proteome are kept in the directory 
    defined by -keep option and the namings is derived based on the name of initial file and number in the list of 
    files. Example:
    
  	PRJNA57623_prodigal.orf.fsa.1.profiles
	PRJNA57623.fsa.1.profiles
    PRJNA57623.proteins.fsa.1.profiles
    
    Example of the header in PRJNA57623.proteins.fsa.1.profiles file:
   		 >PF04303.0 ID="AE003852_CDS_1424656-1425858" Function="PrpF_protein" Description="Vibrio cholerae O1 biovar eltor str. N16961 chromosome I, complete sequence." Pathway="NA"
   		 
   		ID is the name of profile (name of gene family). Each gene family can have more than one member in the same group. 
   		Number in the end of profile indicates the number, which is assigned to the member in the family. 
   		For example, if there are 5 genes in the same family, numbers will follow from 0 to 4. 
   		If profile if made of more than 1 domain => Domain accession numbers are separated by "_" and are sorted alphabetically.
   		If profile starts with PF => it is made of PfamA domains, if starts with TIGR => made of TIGRFAM domains, if with SSF => made of Superfamily domains
   		 
   		Description contains:
   			GeneID, followed by ID sign. GeneID contains Start and End position information; 
   			Predicted function of domais, which profile is made of. If profile is made of more than 1 domain, functions are separated by || and are showed in the same order 
   			as domains in the profile. 
   			Description of the genomes the gene was taken from.
   			Pathways, which domains are associated with. In case pathway information is not available - "NA" is displayed. 
    	
    The program will for each proteome X calculate the follwing:

    - Number of proteins observed in X
    - Number of protein families observed in X
    - Number of new genes observed in genome in X compared to 0 .. (X-1) 
    - Size of pan genome at genome X
    - Size of core genome at genome X

    The program assumes two proteins being similar if they share the same functional profile. 
    
OPTIONS
	-memory
		Ignores interpro scanning and takes the results in the directories defined by -PfamA, -TIGRFAM, -Superfamily
       
	-keep <Dirname>
		Specify the name of the directory where results will be kept. Default is PanFunPro
		
	-type <type>
		Types can be 'assembly', 'genbank', 'proteins'
		
		'assembly' and 'genbank' options require additional prediction of open reading frames. 
   		
   	-l|list <Path+filename>
		Specify the full path to the list of proteomes, which will be analysed
   		
	-f|file <Path+filename>
 		Specify the full path to the proteome (file), which is going to be analysed
   		
	-tbl <filename>
		Specify the filename of the file to which the result table will be written. Default is CorePanResults.txt
   		
	-root <Path>
		Specify the Path of the directory, which will be set as current working directory. Default is set by getcwd() command
   		
	-interpro_scans <Path/InterproScans>
		Specify direcotry where results of Interpro scans are going to be stored
		
	-interpro_path <Path>
    	Specify path to the directory where interpro directory is situated.
    	
	-feed_num <number>
		Specify the number of proteomes that can be scaned by InterproScan simultaniuosly. Default is 1.
    	
	-scripts_path <FullPath>
		Specify the Path of the directory, where scripts are located. For example: -scripts_path /Users/oksana/Scripts
		
	-prodigal_link <Path+prodigal_script>
		Specify the Path of the directory, where prodigal script are located. For example: -prodigal_link /Users/oksana/Scripts/prodigal.v2_60
    	
	-h|help
		Prints out description of the code
 
VERSION
    This version (1.0, August 11, 2013) is the first version of the script.

EXAMPLE
   perl PanFunPro2profiles.pl -list /home/projects/oksana/List.Genomes.cf -memory -keep PanFunPro -scripts_path /Users/oksana/Scripts


AUTHOR
    Oksana Lukjancenko, oksana\@cbs.dtu.dk

END
  exit;
}
